@@ -26,28 +26,21 @@
 #include "baobab.h"
 
 void baobab_get_filesystem (baobab_fs *fs);
-void set_statusbar (const gchar *);
 gchar* dir_select (gboolean, GtkWidget *);
 void on_toggled (GtkToggleButton *, gpointer);
-GdkPixbuf *set_bar (gfloat );
 void check_menu_sens (gboolean);
 void stop_scan (void);
 gboolean show_bars (GtkTreeModel *model,
 		    GtkTreePath *path,
 		    GtkTreeIter *iter,
 		    gpointer data);
-void message (gchar *, GtkWidget *);
-gint messageyesno (gchar *messaggio, GtkWidget *parent);
+void message (gchar *, gchar *, GtkMessageType, GtkWidget *);
+gint messageyesno (gchar *primary_msg, gchar *secondary_msg, GtkMessageType type, gchar * ok_button, GtkWidget *parent);
 gboolean baobab_check_dir (const gchar *);
 void popupmenu_list (GtkTreePath *path, GdkEventButton *event, gboolean is_trash);
-void popupmenu_list_search (GtkTreePath *path, GdkEventButton *event, gboolean is_trash);
 void open_nautilus (GtkMenuItem *, gpointer );
-void dialog_search (void);
-void switch_view (gint);
-gint get_NB_page (void);
 void set_label_scan (baobab_fs *);
-void show_label (gint view);
-void set_label_search (gint , guint64 );
+void show_label (void);
 void open_file_with_application (gchar *file);
 gchar * get_trash_path (const gchar *file);
 gboolean trash_file (const gchar *filename);
