@@ -48,7 +48,7 @@ file_exist (char *filename, gboolean show_error)
    char *secondary = NULL;
 
    if (filename == NULL)
-       return;
+       return FALSE;
 
    result = gnome_vfs_open (&handle, filename, GNOME_VFS_OPEN_READ);
    if (result != GNOME_VFS_OK) {
@@ -83,9 +83,11 @@ file_is_zipped (char *filename)
 	char *mime_type;
 
     if (filename == NULL)
-        return;
+        return FALSE;
 
 	mime_type = gnome_vfs_get_mime_type (filename);
+    if (mime_type == NULL)
+        return FALSE;
 
 	if (strcmp (mime_type, "application/x-gzip")==0 ||
 	    strcmp (mime_type, "application/x-zip")==0 ||
@@ -104,7 +106,7 @@ file_is_log (char *filename, gboolean show_error)
     LogStats *stats;
 
     if (filename == NULL)
-        return;
+        return FALSE;
 
     stats = log_stats_new (filename, show_error);
     if (stats==NULL)
@@ -131,11 +133,14 @@ string_get_date_string (gchar *line)
     gchar *month=NULL, *day=NULL;
     int i=0;
 
-    if (line == NULL)
-        return;
+    if (line == NULL || line[0] == 0)
+        return NULL;
 
     split = g_strsplit (line, " ", 4);
-    while ((day == NULL || month == NULL) && split[i]!=NULL) {
+    if (split == NULL)
+        return NULL;
+
+    while ((day == NULL || month == NULL) && split[i]!=NULL && i<4) {
         if (g_str_equal (split[i], "")) {
             i++;
             continue;
@@ -143,6 +148,10 @@ string_get_date_string (gchar *line)
 
         if (month == NULL) {
             month = split[i++];
+            /* If the first field begins by a number, the date
+               is given in yyyy-mm-dd format */
+            if (!g_ascii_isalpha (month[0]))
+                break;
             continue;
         }
 
@@ -165,11 +174,11 @@ string_get_date_string (gchar *line)
    and are then corrected to the correct year once we
    reach the end.
 */
-GList *
+GSList *
 log_read_dates (gchar **buffer_lines, time_t current)
 {
    int offsetyear = 0, current_year;
-   GList *days = NULL, *days_copy;
+   GSList *days = NULL, *days_copy;
    GDate *date, *newdate;
    struct tm *tmptm;
    gchar *date_string;
@@ -197,7 +206,7 @@ log_read_dates (gchar **buffer_lines, time_t current)
 
    g_date_set_year (date, current_year);
    day = g_new (Day, 1);
-   days = g_list_append (days, day);
+   days = g_slist_append (days, day);
 
    day->date = date;
    day->first_line = i;
@@ -236,43 +245,53 @@ log_read_dates (gchar **buffer_lines, time_t current)
        g_free (date_string);
 
        if (!done) {
+         /* We need to find the first line now that has a date
+            Logs can have some messages without dates ... */
+         newdate = NULL;
+         while (newdate == NULL && !done) {
            i++;
            date_string = string_get_date_string (buffer_lines[i]);
            if (date_string == NULL)
-               continue;
+             continue;
            newdate = string_get_date (buffer_lines[i]);
-           if (newdate == NULL)
-               continue;      
+           
+           if (newdate == NULL && i==n-1)
+             done = TRUE;
+         }
 
-           /* Append a day to the list */	
+         day->last_line = i-1;
+
+         /* Append a day to the list */	
+         if (newdate) {
            g_date_set_year (newdate, current_year + offsetyear);	
            if (g_date_compare (newdate, date) < 1) {
-               offsetyear++; /* newdate is next year */
-               g_date_add_years (newdate, 1);
+             offsetyear++; /* newdate is next year */
+             g_date_add_years (newdate, 1);
            }
            
            date = newdate;
            day = g_new (Day, 1);
-           days = g_list_append (days, day);
+           days = g_slist_append (days, day);
            
            day->date = date;
            day->first_line = i;
            day->last_line = -1;
            rangemin = i;
            rangemax = n;
+         }
        }
    }
 
    /* Correct years now. We assume that the last date on the log
       is the date last accessed */
 
-   for (days_copy = days; days_copy != NULL; days_copy = g_list_next (days_copy)) {       
+   for (days_copy = days; days_copy != NULL; days_copy = g_slist_next (days_copy)) {       
        day = days_copy -> data;
        g_date_subtract_years (day->date, offsetyear);
    }
    
    /* Sort the days in chronological order */
-   days = g_list_sort (days, days_compare);
+   days = g_slist_sort (days, days_compare);
 
    return (days);
 }
@@ -290,7 +309,7 @@ log_stats_new (char *filename, gboolean show_error)
    GnomeVFSFileInfo *info;
    GnomeVFSHandle *handle;
    GnomeVFSFileSize size;
-   LogStats *lstats;
+   LogStats *stats;
    char buff[1024];
    char *found_space;
 
@@ -320,92 +339,95 @@ log_stats_new (char *filename, gboolean show_error)
    
    found_space = g_strstr_len (buff, 1024, " ");
    if (found_space == NULL) {
-	   if (show_error) {
-		   g_snprintf (buff, sizeof (buff), _("%s not a log file."), filename);
-		   error_dialog_show (NULL, error_main, buff);
-	   }
        gnome_vfs_file_info_unref (info);
 	   return NULL;
    }
    
-   lstats = g_new (LogStats, 1);   
-   lstats->mtime = info->mtime;
-   lstats->size = info->size;
+   stats = g_new (LogStats, 1);   
+   stats->file_time = info->mtime;
+   stats->file_size = info->size;
    gnome_vfs_file_info_unref (info);
 
-   return (lstats);
+   return (stats);
 }
 
 Log *
 log_open (char *filename, gboolean show_error)
 {
-   Log *log;
-   char *buffer, *buffer2;
-   char *display_name = NULL;
-   LogStats *stats;
-   int i, j;
-   GList *days;
-   Day *day;
-   GError *error;
+   char *buffer, *zipped_name= NULL, *error_message;
    GnomeVFSResult result;
-   ulong size;
+   LogStats *stats;
+   gboolean opened = TRUE;
+   int i, size;
+   GList *days;
+   Log *log;
    
-   if (file_exist (filename, show_error) == FALSE)
-	   return NULL;
-
-   if (file_is_zipped (filename)) {
-	   display_name = filename;
-	   filename = g_strdup_printf ("%s#gzip:", display_name);
-   }   
-
    stats = log_stats_new (filename, show_error);
-   if (stats == NULL)
-       return NULL;
-
-   result = gnome_vfs_read_entire_file (filename, &size, &buffer);
-   buffer[size-1] = 0;
-   if (result != GNOME_VFS_OK) {
-	   error_dialog_show (NULL, error_main, _("Unable to open logfile!\n"));
-	   return NULL;
+   if (stats == NULL) {
+       if (file_is_zipped (filename)) {    
+           zipped_name = g_strdup_printf ("%s#gzip:", filename);
+           stats = log_stats_new (filename, show_error);
+           if (stats == NULL) {
+               opened = FALSE;
+           }
+       } else
+           opened = FALSE;
+   }
+   
+   if (opened == FALSE) {
+       error_message = g_strdup_printf (_("%s is not a log file."), filename);
+       goto error;
    }
 
    log = g_new0 (Log, 1);   
    if (log == NULL) {
-	   error_dialog_show (NULL, error_main, _("Not enough memory!\n"));
-	   return NULL;
+       error_message = g_strdup (_("Not enough memory."));
+       goto error;
+   }
+
+   if (!zipped_name) {
+       log->name = g_strdup (filename);
+   } else {
+       log->name = zipped_name;
+       log->display_name = g_strdup (filename);
+   }
+
+   result = gnome_vfs_read_entire_file (log->name, &size, &buffer);
+   buffer[size-1] = 0;
+   if (result != GNOME_VFS_OK) {
+       error_message = g_strdup_printf (_("%s cannot be opened."), log->name);
+       goto error;
    }
-   log->name = g_strdup (filename);
-   log->display_name = display_name;
-   if (display_name)
-	   g_free (filename);
 
    if (g_get_charset (NULL) == FALSE) {
+       char *buffer2;
        buffer2 = locale_to_utf8 (buffer);
        g_free (buffer);
        buffer = buffer2;
    }
 
    log->lines = g_strsplit (buffer, "\n", -1);
-   g_free (buffer2);
+   g_free (buffer);   
 
    log->total_lines = g_strv_length (log->lines);
-   log->displayed_lines = 0;
+   log->displayed_lines = log->total_lines;
    log->first_time = TRUE;
-   log->lstats = stats;
+   log->stats = stats;
    log->model = NULL;
    log->filter = NULL;
+   log->mon_offset = 0;
+   log->bold_rows_list = NULL;
 
    /* A log without dates will return NULL */
-   log->days = log_read_dates (log->lines, log->lstats->mtime);
+   log->days = log_read_dates (log->lines, log->stats->file_time);
 
    /* Check for older versions of the log */
    log->versions = 0;
    log->current_version = 0;
    log->parent_log = NULL;
-   log->mon_offset = size;
    for (i=1; i<5; i++) {
        gchar *older_name;
-       older_name = g_strdup_printf ("%s.%d", filename, i);
+       older_name = g_strdup_printf ("%s.%d", log->name, i);
        log->older_logs[i] = log_open (older_name, FALSE);
        g_free (older_name);
        if (log->older_logs[i] != NULL) {
@@ -418,6 +440,17 @@ log_open (char *filename, gboolean show_error)
    }
 
    return log;
+
+   /* Error catching */
+
+ error:
+
+   if (error_message && show_error) {
+       error_dialog_show (NULL, error_main, error_message);
+       g_free (error_message);
+   }
+   return NULL;
+   
 }
 
 static void
@@ -429,7 +462,7 @@ log_add_lines (Log *log, gchar *buffer)
   g_assert (buffer != NULL);
 
   old_buffer = g_strjoinv ("\n", log->lines);
-  new_buffer = g_strconcat (old_buffer, buffer);
+  new_buffer = g_strconcat (old_buffer, "\n", buffer, NULL);
   g_free (old_buffer);
   
   g_strfreev (log->lines);
@@ -439,35 +472,51 @@ log_add_lines (Log *log, gchar *buffer)
   log->total_lines = g_strv_length (log->lines);
 }
 
+void log_stats_reload (Log *log)
+{
+  g_free (log->stats);
+  log->stats = log_stats_new (log->name, TRUE);
+}
+
 /* log_read_new_lines */
 
 gboolean 
 log_read_new_lines (Log *log)
 {
-	GnomeVFSResult result;
-	gchar *buffer;
-    GnomeVFSFileSize newsize, read;
-    GnomeVFSFileOffset size;
-
-	g_return_val_if_fail (log!=NULL, FALSE);
+    GnomeVFSResult result;
+    gchar *buffer;
+    GnomeVFSFileSize newfilesize, read;
+    guint64 size, newsize;
+    
+    g_return_val_if_fail (log!=NULL, FALSE);
     
     result = gnome_vfs_seek (log->mon_file_handle, GNOME_VFS_SEEK_END, 0L);
-	result = gnome_vfs_tell (log->mon_file_handle, &newsize);
+    result = gnome_vfs_tell (log->mon_file_handle, &newfilesize);
     size = log->mon_offset;
-    
-	if (newsize > log->mon_offset) {
-        buffer = g_malloc (newsize-size);
-        result = gnome_vfs_seek (log->mon_file_handle, GNOME_VFS_SEEK_START, size);
-        result = gnome_vfs_read (log->mon_file_handle, buffer, newsize-size, &read);
-        buffer [newsize-size-1] = 0;
-        log->mon_offset = newsize;
-        
-        log_add_lines (log, buffer);
-        g_free (buffer);
+    newsize = newfilesize;
 
-        return TRUE;
-	}
+    if (newsize > size) {
+      buffer = g_malloc (newsize - size);
+      if (!buffer)
 	return FALSE;
+
+      result = gnome_vfs_seek (log->mon_file_handle, GNOME_VFS_SEEK_START, size);
+      if (result != GNOME_VFS_OK)
+	return FALSE;
+
+      result = gnome_vfs_read (log->mon_file_handle, buffer, newsize-size, &read);
+      if (result != GNOME_VFS_OK)
+	return FALSE;
+
+      buffer [newsize-size-1] = 0;
+      log->mon_offset = newsize;      
+      log_add_lines (log, buffer);
+      g_free (buffer);
+      
+      log_stats_reload (log);
+      return TRUE;
+    }
+    return FALSE;
 }
 
 /* 
@@ -482,7 +531,7 @@ log_unbold (gpointer data)
     LogList *list;
     Log *log = data;
 
-    g_return_if_fail (log != NULL);
+    g_return_val_if_fail (log != NULL, FALSE);
 
     logview = log->window;
 
@@ -500,7 +549,7 @@ log_close (Log *log)
 {
    gint i;
    Day *day;
-   GList *days;
+   GSList *days;
 
    g_return_if_fail (log);
    
@@ -518,20 +567,51 @@ log_close (Log *log)
    g_strfreev (log->lines);
 
    if (log->days != NULL) {
-       for (days = log->days; days != NULL; days = g_list_next (days)) {
+       for (days = log->days; days != NULL; days = g_slist_next (days)) {
            day = days->data;
            g_date_free (day->date);
            gtk_tree_path_free (day->path);
            g_free (day);
        }
-       g_list_free (log->days);
+       g_slist_free (log->days);
        log->days = NULL;
    }
    
-   g_free (log->lstats);
-   gtk_tree_path_free (log->current_path);
-   log->current_path = NULL;
+   g_free (log->stats);
+
+   if (log->selected_range.first)
+     gtk_tree_path_free (log->selected_range.first);
+   if (log->selected_range.last)
+     gtk_tree_path_free (log->selected_range.last);
+   if (log->visible_first)
+     gtk_tree_path_free (log->visible_first);
 
    g_free (log);
    return;
 }
+
+gchar *
+log_extract_filename (Log *log)
+{
+  GnomeVFSURI *uri;
+  gchar *filename;
+
+  uri = gnome_vfs_uri_new (log->name);
+  filename = gnome_vfs_uri_extract_short_name (uri);
+  gnome_vfs_uri_unref (uri);
+
+  return filename;
+}
+
+gchar *
+log_extract_dirname (Log *log)
+{
+  GnomeVFSURI *uri;
+  gchar *dirname;
+
+  uri = gnome_vfs_uri_new (log->name);
+  dirname = gnome_vfs_uri_extract_dirname (uri);
+  gnome_vfs_uri_unref (uri);
+
+  return dirname;
+}
